package ami

import (
	"context"
	"errors"
	"fmt"
	"strings"
	"time"

	goami "gitlab.atsaero.ru/go-packages/goami/ami"
)

type Ami interface {
	MixMonitor(tag, channelID, file string, clbFunc ...CallbackEventFunc) error
	Monitor(tag, channelID, file, format string, mix bool, clbFunc ...CallbackEventFunc) error
	StopMonitor(tag, channelID string, clbFunc ...CallbackEventFunc) error
	Redirect(tag, channelID, ext string, clbFunc ...CallbackEventFunc) error
	Setvar(tag, channelID, varl, value string, clbFunc ...CallbackEventFunc) error
	Originate(tag, channelID, ext, cnt, variables string, timeout int, clbFunc ...CallbackEventFunc) error
	Ping(tag string, clbFunc ...CallbackEventFunc) error
}

func (p *connPool) Ping(tag string, clbFunc ...CallbackEventFunc) error {
	uuid, err := goami.GetUUID()
	if err != nil {
		return fmt.Errorf("%s GetUUID failed: %v", tag, err)
	}

	uuid += `-ping`

	commFunc := p.launchCommands(clbFunc...)

	comm := func(socket *goami.Socket) error {
		ctx, cancel := context.WithTimeout(context.Background(), time.Millisecond*500)
		defer cancel()

		err = goami.Ping(ctx, socket, uuid)
		if err != nil {
			return fmt.Errorf("%s [%s] %v", tag, socket.RemoteHost(), err)
		}

		p.logger.Infof("%s [%s] Ping successfull", tag, socket.RemoteHost())

		return nil
	}

	return commFunc(comm, uuid)
}

func (p *connPool) MixMonitor(tag, channelID, file string, clbFunc ...CallbackEventFunc) error {
	uuid, err := goami.GetUUID()
	if err != nil {
		return fmt.Errorf("%s GetUUID failed: %v", tag, err)
	}

	uuid += `-mixMonitor`

	commFunc := p.launchCommands(clbFunc...)

	comm := func(socket *goami.Socket) error {
		ctx, cancel := context.WithTimeout(context.Background(), time.Millisecond*500)
		defer cancel()

		res, err := goami.MixMonitor(ctx, socket, uuid, channelID, file, "", "")
		if err != nil {
			return fmt.Errorf("%s [%s] %v", tag, socket.RemoteHost(), err)
		}

		p.logger.Infof("%s [%s] MixMonitor successfull", tag, socket.RemoteHost())

		if strings.ToLower(res.Get("Response")) == "error" {
			msg := res.Get("Message")
			if msg == "" {
				return errors.New("unknown error")
			}
			return errors.New(msg)
		}

		return nil
	}

	return commFunc(comm, uuid)
}

func (p *connPool) Monitor(tag, channelID, file, format string, mix bool, clbFunc ...CallbackEventFunc) error {
	uuid, err := goami.GetUUID()
	if err != nil {
		return fmt.Errorf("%s GetUUID failed: %v", tag, err)
	}

	uuid += `-monitor`

	commFunc := p.launchCommands(clbFunc...)

	comm := func(socket *goami.Socket) error {
		ctx, cancel := context.WithTimeout(context.Background(), time.Millisecond*500)
		defer cancel()

		res, err := goami.Monitor(ctx, socket, uuid, channelID, file, format, mix)
		if err != nil {
			return fmt.Errorf("%s [%s] %v", tag, socket.RemoteHost(), err)
		}

		p.logger.Infof("Monitor successfull at host %s", socket.RemoteHost())

		if strings.ToLower(res.Get("Response")) == "error" {
			msg := res.Get("Message")
			if msg == "" {
				return errors.New("unknown error")
			}
			return errors.New(msg)
		}

		return nil
	}

	return commFunc(comm, uuid)

}

func (p *connPool) StopMonitor(tag, channelID string, clbFunc ...CallbackEventFunc) error {
	uuid, err := goami.GetUUID()
	if err != nil {
		return fmt.Errorf("%s GetUUID failed: %v", tag, err)
	}

	uuid += `-stopMonitor`

	commFunc := p.launchCommands(clbFunc...)

	comm := func(socket *goami.Socket) error {
		ctx, cancel := context.WithTimeout(context.Background(), time.Millisecond*500)
		defer cancel()

		res, err := goami.StopMonitor(ctx, socket, uuid, channelID)
		if err != nil {
			return fmt.Errorf("%s [%s] %v", tag, socket.RemoteHost(), err)
		}

		p.logger.Infof("%s [%s] StopMonitor successfull", tag, socket.RemoteHost())

		if strings.ToLower(res.Get("Response")) == "error" {
			msg := res.Get("Message")
			if msg == "" {
				return errors.New("unknown error")
			}
			return errors.New(msg)
		}

		return nil
	}

	return commFunc(comm, uuid)

}

func (p *connPool) Setvar(tag, channelID, varl, value string, clbFunc ...CallbackEventFunc) error {
	uuid, err := goami.GetUUID()
	if err != nil {
		return fmt.Errorf("%s GetUUID failed: %v", tag, err)
	}

	uuid += `-setvar`

	commFunc := p.launchCommands(clbFunc...)

	comm := func(socket *goami.Socket) error {
		ctx, cancel := context.WithTimeout(context.Background(), time.Millisecond*500)
		defer cancel()

		res, err := goami.Setvar(ctx, socket, uuid, channelID, varl, value)
		if err != nil {
			return fmt.Errorf("%s [%s] %v", tag, socket.RemoteHost(), err)
		}

		p.logger.Infof("%s [%s] Setvar successfull", tag, socket.RemoteHost())

		if strings.ToLower(res.Get("Response")) == "error" {
			msg := res.Get("Message")
			if msg == "" {
				return errors.New("unknown error")
			}
			return errors.New(msg)
		}

		return nil
	}

	return commFunc(comm, uuid)
}

func (p *connPool) Redirect(tag, channelID, ext string, clbFunc ...CallbackEventFunc) error {
	uuid, err := goami.GetUUID()
	if err != nil {
		return fmt.Errorf("%s GetUUID failed: %v", tag, err)
	}

	uuid = uuid + `-redirect`

	commFunc := p.launchCommands(clbFunc...)

	comm := func(socket *goami.Socket) error {
		ctx, cancel := context.WithTimeout(context.Background(), time.Millisecond*500)
		defer cancel()
		callData := goami.CallData{
			Channel:  channelID,
			Exten:    ext,
			Context:  "transfer",
			Priority: "1",
		}
		res, err := goami.Redirect(ctx, socket, uuid, callData)
		if err != nil {
			return fmt.Errorf("%s [%s] %v", tag, socket.RemoteHost(), err)
		}

		p.logger.Infof("%s [%s] Redirect successfull", tag, socket.RemoteHost())

		if strings.ToLower(res.Get("Response")) == "error" {
			msg := res.Get("Message")
			if msg == "" {
				return errors.New("unknown error")
			}
			return errors.New(msg)
		}

		return nil
	}

	return commFunc(comm, uuid)
}

func (p *connPool) Originate(tag, channelID, ext, cnt, variables string, timeout int, clbFunc ...CallbackEventFunc) error {
	uuid, err := goami.GetUUID()
	if err != nil {
		return fmt.Errorf("%s GetUUID failed: %v", tag, err)
	}

	uuid = uuid + `-originate`

	commFunc := p.launchCommands(clbFunc...)

	comm := func(socket *goami.Socket) error {
		ctx, cancel := context.WithTimeout(context.Background(), time.Second*5)
		defer cancel()

		callData := goami.OriginateData{
			Channel:  channelID,
			Exten:    ext,
			Context:  cnt,
			Priority: 1,
			Timeout:  timeout,
			Account:  uuid,
			Async:    "true",
			Variable: variables,
		}

		res, err := goami.Originate(ctx, socket, uuid, callData)
		if err != nil {
			return fmt.Errorf("%s [%s] %v", tag, socket.RemoteHost(), err)
		}

		p.logger.Infof("%s [%s] Originate successfull", tag, socket.RemoteHost())

		if strings.ToLower(res.Get("Response")) == "error" {
			msg := res.Get("Message")
			if msg == "" {
				return errors.New("unknown error")
			}
			return errors.New(msg)
		}

		return nil
	}

	return commFunc(comm, uuid)
}
