package ami

import (
	"bytes"
	"context"
	"fmt"
	"strings"
	"sync"
	"sync/atomic"
	"time"

	"github.com/sirupsen/logrus"
	goami "gitlab.atsaero.ru/go-packages/goami/ami"
)

const (
	EventDialBegin         = "DialBegin"
	EventDialEnd           = "DialEnd"
	EventHangup            = "Hangup"
	EventOriginateResponse = "OriginateResponse"
	EventUserEvent         = "UserEvent"
	EventError             = "Error"
)

const (
	checkPeriod        = time.Minute * 5  // период запуска checker
	cleanePeriod       = time.Minute * 17 // период запуска cleaner
	callbackTimeToLive = time.Hour        // лимит жизни callback
	rights             = "call,all"
)

// CallbackEventFunc функция обработчика события, возвращает флаг завершения коллбека и ошибку
type CallbackEventFunc func(EventResponse) (bool, error)

// callback элемент очереди обработчика события
type callback struct {
	closed      atomic.Bool
	f           CallbackEventFunc
	timeCreated time.Time
}

// event структура события
type event struct {
	key  string
	data EventResponse
}

type eventPool struct {
	sync.RWMutex
	avail              atomic.Bool
	callbacks          map[string]*callback
	callbacksForRemove map[string]struct{}
	events             chan event
	ctx                context.Context
	addr               string
	user               string
	secret             string
	logger             *logrus.Entry
}

type EventResponse map[string][]string

func (er EventResponse) Get(key string) string {
	if er == nil {
		return ""
	}
	rs := er[key]
	if len(rs) == 0 {
		return ""
	}
	return rs[0]
}

func (er EventResponse) Event() string {
	return er.Get("Event")
}

func (er EventResponse) Key() string {
	return er.Get("KEY")
}

func (er EventResponse) Error() string {
	return er.Get("Error")
}

func newEventPool(ctx context.Context, addr, user, secret string, logger *logrus.Entry) *eventPool {
	e := eventPool{
		ctx:                ctx,
		logger:             logger,
		callbacks:          make(map[string]*callback),
		callbacksForRemove: make(map[string]struct{}),
		events:             make(chan event, 10000),
		addr:               addr,
		user:               user,
		secret:             secret,
	}

	// запуск обработчика событий
	go e.launchHandler()

	// запуск механизма удаления отработавших callbacks
	go func() {
		t := time.NewTicker(cleanePeriod)
		tc := time.NewTicker(checkPeriod)

		for e.ctx.Err() == nil {
			select {
			case <-tc.C:
				num := e.check()
				e.logger.Debugf("calbacks checker: %d callbacks for delete found", num)
			case <-t.C:
				num := e.clean()
				e.logger.Debugf("calbacks cleaner: %d callbacks deleted", num)
			}
		}
	}()

	// запуск worker
	launchDuration := time.Millisecond
	go func() {
		for e.ctx.Err() == nil {
			time.Sleep(launchDuration)
			if err := e.worker(); err != nil {
				e.logger.Errorf("start event worker for '%s' failed: %v", e.addr, err)
			}
			if launchDuration < time.Minute*5 {
				launchDuration += time.Second * 10
			}
		}
	}()

	return &e
}

func (e *eventPool) isAvail() bool {
	return e.avail.Load()
}

func (e *eventPool) launchHandler() {
	e.logger.Debug("eventPool handler started")
	defer e.logger.Debug("eventPool handler stopped")

	for {
		select {
		case <-e.ctx.Done():
			return
		case event, ok := <-e.events:
			if !ok {
				return
			}
			e.logger.Debugf("got event %+v", event)

			e.RLock()
			calback, ok := e.callbacks[event.key]
			e.RUnlock()

			if ok {
				go func(c *callback) {
					e.logger.Debugf("event '%s' start processing", event.key)
					defer func() {
						e.logger.Debugf("event '%s' stop processing", event.key)
					}()

					if c.closed.Load() {
						return
					}

					needRemove, err := c.f(event.data)
					if err != nil {
						e.logger.Errorf("ami event '%s' failed: %v", event.key, err)
					}

					if needRemove {
						c.closed.Store(true)
					}
				}(calback)
			}

		}
	}
}

// check() выявляет завершенные и протухших задания и помещает их в очередь на удаление
func (e *eventPool) check() int {
	var num int
	e.RLock()
	for key, cb := range e.callbacks {
		if cb.closed.Load() || time.Now().After(cb.timeCreated.Add(callbackTimeToLive)) {
			e.callbacksForRemove[key] = struct{}{}
			num++
		}
	}
	e.RUnlock()

	return num
}

// clean очищает очередь calbacks
func (e *eventPool) clean() int {
	keysForDel := make([]string, 0)
	e.RLock()
	for key := range e.callbacksForRemove {
		keysForDel = append(keysForDel, key)
	}
	e.RUnlock()
	if len(keysForDel) > 0 {
		e.Lock()
		for _, key := range keysForDel {
			delete(e.callbacks, key)
		}
		e.Unlock()
	}

	return len(keysForDel)
}

func (e *eventPool) AddCallback(f CallbackEventFunc, key string) {
	e.Lock()
	defer e.Unlock()
	e.callbacks[key] = &callback{f: f, timeCreated: time.Now()}
}

func (e *eventPool) PushEvent(ev event) {
	select {
	case e.events <- ev:
	default:
		e.logger.Warnf("skip event '%s': chan overload", ev.key)
	}
}

func (e *eventPool) PushEventError(key, errText string) {
	ev := event{key: key, data: EventResponse{"Event": []string{EventError}, "KEY": []string{key}, "Error": []string{errText}}}
	select {
	case e.events <- ev:
	default:
		e.logger.Warnf("skip event '%s': chan overload", ev.key)
	}
}

func (e *eventPool) worker() error {
	e.logger.Debugf("start event-worker for %s", e.addr)
	defer func() {
		e.avail.Store(false)
		e.logger.Debugf("stopped event-worker for %s", e.addr)
	}()

	ctxSocket, cancelSocket := context.WithCancel(e.ctx)
	defer cancelSocket()

	connTimer := time.NewTimer(dialTimeout)
	connected := make(chan struct{})

	// реализация dialTimeout для ami.NewSocket
	go func() {
		select {
		case <-connTimer.C:
			cancelSocket()
		case <-connected:
		}
	}()

	uuid, err := goami.GetUUID()
	if err != nil {
		return fmt.Errorf("ami.GetUUID failed: %v", err)
	}

	socket, err := goami.NewSocket(ctxSocket, e.addr)
	if err != nil {
		return fmt.Errorf("ami.NewSocket failed: %v", err)
	}
	defer socket.Close(ctxSocket)

	if err := goami.Login(ctxSocket, socket, e.user, e.secret, rights, uuid); err != nil {
		return fmt.Errorf("ami.Login failed: %v", err)
	}

	close(connected)

	e.logger.Infof("event-worker for %s connected and ready", e.addr)
	e.avail.Store(true)

	var buffer bytes.Buffer
	for e.ctx.Err() == nil {
		input, err := socket.Recv(ctxSocket)
		if err != nil {
			return fmt.Errorf("socket.Recv failed: %v", err)
		}
		buffer.WriteString(input)
		if !strings.HasSuffix(buffer.String(), "\r\n\r\n") {
			continue
		}
		ev, err := parseEvent(buffer.String())
		if err != nil {
			e.logger.Errorf("parseEvent failed: %v", err)
			continue
		}
		key := getKey(ev)
		ev["KEY"] = []string{key}

		e.PushEvent(event{key: key, data: ev})
		buffer.Reset()
	}

	return nil
}

func parseEvent(input string) (EventResponse, error) {
	resp := make(EventResponse)
	lines := strings.Split(input, "\r\n")
	for _, line := range lines {
		keys := strings.SplitAfterN(line, ":", 2)
		if len(keys) == 2 {
			key := strings.TrimSpace(strings.Trim(keys[0], ":"))
			value := strings.TrimSpace(keys[1])
			resp[key] = append(resp[key], value)
		} else if strings.Contains(line, "\r\n\r\n") || line == "" {
			return resp, nil
		}
	}
	return resp, nil
}

func getKey(ev EventResponse) string {
	var key string
	switch ev.Get("Event") {
	case EventDialBegin:
		key = ev.Get("DestAccountCode")
	case EventDialEnd:
		key = ev.Get("DestAccountCode")
	case EventHangup:
		key = ev.Get("AccountCode")
	case EventOriginateResponse:
		key = ev.Get("ActionID")
	case EventUserEvent:
		key = ev.Get("AccountCode")
	}

	return key
}
