package ami

import (
	"context"
	"fmt"
	"reflect"
	"regexp"
	"time"

	"github.com/sirupsen/logrus"
	goami "gitlab.atsaero.ru/go-packages/goami/ami"
)

const (
	dialTimeout = time.Second * 5  // таймаут соединения с астериском
	liveTime    = time.Minute * 30 // время жизни воркера
)

// commandFunc тип функции-обработчика ami команды
type commandFunc func(*goami.Socket) error

type poolItemParams struct {
	f commandFunc
	c []CallbackEventFunc
	k string
}

// connPool пул соединений
type connPool struct {
	ch     chan poolItemParams
	pool   []*poolItem
	ctx    context.Context
	cancel context.CancelFunc
	logger *logrus.Entry
	user   string
	secret string
}

var ADDRTEMPLATE = regexp.MustCompile(`^[^:]+:[\d]+$`)

func NewConnPool(ctx context.Context, user, secret string, workersAmount, cps int, logger *logrus.Entry, address ...string) Ami {
	ctx, cancel := context.WithCancel(ctx)
	c := &connPool{
		ch:     make(chan poolItemParams),
		ctx:    ctx,
		cancel: cancel,
		user:   user,
		secret: secret,
		logger: logger.WithField("model", "pool"),
	}

	for _, addr := range address {
		if !ADDRTEMPLATE.MatchString(addr) {
			logger.Errorf("skip address '%s': wrong format", addr)
			continue
		}
		c.pool = append(c.pool, newPoolItem(ctx, addr, user, secret, workersAmount, cps, logger))
	}

	if len(c.pool) == 0 {
		c.cancel()
		return c
	}

	go func() {
		cases := make([]reflect.SelectCase, len(c.pool)+1)
		cases[len(c.pool)] = reflect.SelectCase{Dir: reflect.SelectRecv, Chan: reflect.ValueOf(c.ctx.Done())}
		for ctx.Err() == nil {
			select {
			case <-ctx.Done():
			case param := <-c.ch:
				for i, poolItem := range c.pool {
					cases[i] = reflect.SelectCase{Dir: reflect.SelectSend, Chan: reflect.ValueOf(poolItem.ch), Send: reflect.ValueOf(param)}
				}
				reflect.Select(cases)
			}
		}
	}()

	return c
}

// launchCommands возвращает функцию-приемник для обработчика команды и коллбэков событий
func (c *connPool) launchCommands(cbs ...CallbackEventFunc) func(commandFunc, string) error {
	return func(fn commandFunc, uuid string) error {
		if fn == nil {
			return fmt.Errorf("empty function for uuid %v", uuid)
		}

		select {
		case <-c.ctx.Done():
			return fmt.Errorf("cancelled for uuid %v", uuid)
		case c.ch <- poolItemParams{f: fn, c: cbs, k: uuid}:
		}

		return nil
	}
}
