package ami

import (
	"context"
	"errors"
	"fmt"
	"sync/atomic"
	"time"

	"github.com/sirupsen/logrus"
	goami "gitlab.atsaero.ru/go-packages/goami/ami"
)

type workerParams struct {
	f commandFunc
	k string
}

// poolItem элемент пула отвечающий за соединения к одному адресу
type poolItem struct {
	cps       int
	ch        chan poolItemParams
	cf        chan workerParams
	wr        *atomic.Int32
	ctx       context.Context
	eventPool *eventPool
	logger    *logrus.Entry
}

func newPoolItem(ctx context.Context, addr, user, secret string, workercAmount, cps int, logger *logrus.Entry) *poolItem {
	p := poolItem{
		ctx:       ctx,
		cps:       cps,
		ch:        make(chan poolItemParams),
		cf:        make(chan workerParams),
		wr:        &atomic.Int32{},
		eventPool: newEventPool(ctx, addr, user, secret, logger),
		logger:    logger,
	}

	go func() {
		t := time.NewTicker(time.Second * 2)
		for p.ctx.Err() == nil {
			for i := 1; i <= workercAmount-int(p.wr.Load()); i++ {
				go func() {
					if err := p.worker(p.cf, addr, user, secret, logger); err != nil {
						p.logger.Errorf("start worker %s failed: %v", addr, err)
					}
				}()
			}
			<-t.C
		}
	}()

	go func() {
		for p.ctx.Err() == nil {
			select {
			case <-p.ctx.Done():
			case param, ok := <-p.ch:
				if !ok {
					break
				}

				// ждем подключения events-worker
				for !p.eventPool.isAvail() {
					time.Sleep(time.Millisecond * 500)
				}

				delyCPS := 1 / float32(p.cps) * 1000

				time.Sleep(time.Millisecond * time.Duration(delyCPS))

				for _, cb := range param.c {
					p.eventPool.AddCallback(cb, param.k)
				}

				select {
				case <-p.ctx.Done():
				case p.cf <- workerParams{f: param.f, k: param.k}:
				}
			}
		}
	}()

	return &p
}

// worker процесс-обработчик соединения
func (p *poolItem) worker(params chan workerParams, addr, user, secret string, logger *logrus.Entry) error {
	p.wr.Add(1)
	defer p.wr.Add(-1)

	// пока событийный сокет не готов, нет смысла стартовать воркер
	if !p.eventPool.isAvail() {
		return errors.New("events socket is not ready yet")
	}

	logger = logger.WithField("addr", addr)

	logger.Debugf("start worker %s", addr)
	defer logger.Debugf("stopped worker %s", addr)

	ctxSocket, cancelSocket := context.WithCancel(context.Background())
	defer cancelSocket()

	connTimer := time.NewTimer(dialTimeout)
	connected := make(chan struct{})

	// реализация dialTimeout для ami.NewSocket
	go func() {
		select {
		case <-connTimer.C:
			cancelSocket()
		case <-connected:
		}
	}()

	uuid, err := goami.GetUUID()
	if err != nil {
		return fmt.Errorf("ami.GetUUID failed: %v", err)
	}

	socket, err := goami.NewSocket(ctxSocket, addr)
	if err != nil {
		return fmt.Errorf("ami.NewSocket failed: %v", err)
	}
	defer socket.Close(ctxSocket)

	if err := goami.Login(ctxSocket, socket, user, secret, "Off", uuid); err != nil {
		return fmt.Errorf("ami.Login failed: %v", err)
	}

	close(connected)

	logger.Infof("worker %s connected and ready", addr)
loop:
	for p.ctx.Err() == nil {

		// событийный сокет закрылся - выходим из воркера
		if !p.eventPool.isAvail() {
			p.logger.Error("events socket is closed")
			break
		}

		select {
		case <-p.ctx.Done():
		case param, ok := <-params:
			if !ok {
				p.logger.Error("command channel is closed")
				break loop
			}

			if param.f == nil || len(param.k) == 0 {
				p.logger.Error("command failed: wrong format")
				break
			}
			if err := param.f(socket); err != nil {
				p.eventPool.PushEventError(param.k, err.Error())
				break loop
			}
		}
	}

	return nil
}
